﻿Imports System.Drawing.Printing
Imports System.Windows.Forms.DataVisualization.Charting.Chart
Imports System.Windows.Forms.DataVisualization.Charting.ChartImageFormat

Public Class frmDiagramm
	Private weatherDataList As List(Of WeatherDataDiag)
	Private imageList As New List(Of Image)()
	Private ReadOnly _datafolder As String = Path.Combine(Application.StartupPath, "ApSet.xml")
	Private NewDatafolder As String
	Private NewLocation As String
	Private NewUser As String
	Private Paperformat As Boolean
	Private Const chartMargin As Integer = 20
	Private ReadOnly Images As New List(Of Image)

#Region "AppDaten holen"
	Private Sub ReadDataFromXml()
		Dim xDoc As XDocument = XDocument.Load(_datafolder)
		' Finde das Element mit dem Namen "dataFolder"
		Dim Datafolder As XElement = xDoc.Root.Element("dataFolder")
		Dim Datalocation As XElement = xDoc.Root.Element("location")
		Dim Datauser As XElement = xDoc.Root.Element("username")
		If Datafolder IsNot Nothing And Datauser IsNot Nothing And Datalocation IsNot Nothing Then
			NewDatafolder = Datafolder.Value & "\"
			NewLocation = Datalocation.Value
			NewUser = Datauser.Value
		Else
			' Zeige eine Fehlermeldung an, wenn das Element nicht gefunden wurde
			MessageBox.Show("Daten konnten nicht gefunden werden.", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error)
		End If
	End Sub
#End Region


#Region "Charts erstellen"
	Private Sub dtp_Show_ValueChanged(sender As Object, e As EventArgs) Handles dtp_Show.ValueChanged
		'Datum im Format "yyyy-MM" aus dem DTPicker auslesen
		Dim selectedDate As String = dtp_Show.Value.ToString("yyyy-MM")
		'Pfad zur ausgewählten XML-Datei erstellen
		Dim xmlFilePath As String = Path.Combine(NewDatafolder, selectedDate & ".xml")
		'Prüfen, ob die ausgewählte XML-Datei existiert
		If File.Exists(xmlFilePath) Then
			Dim weatherDataList As List(Of WeatherDataDiag) = ReadWeatherDataFromXml(xmlFilePath)
			' Chart erstellen
			CreateChart(weatherDataList)
		Else
			MessageBox.Show("Die ausgewählte Datei wurde nicht gefunden.", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error)
		End If
	End Sub

	Private Sub CreateChart(weatherDataList As List(Of WeatherDataDiag))
		' Chart-Serie erstellen
		lblEinheiten.Text = "Regen: Niederschläge in mm"
		Dim chartSeries As Series = New Series("RegenTag")
		chartSeries.Color = Color.Blue
		chartSeries.ChartType = SeriesChartType.Column
		For Each weatherData As WeatherDataDiag In weatherDataList
			chartSeries.Points.AddXY(weatherData.Datum, weatherData.RegenTag)
		Next
		' Chart mit den Serien füllen
		Dim selectedDatein As Date = dtp_Show.Value
		Dim formattedDate As String = selectedDatein.ToString("MMMM-yyyy")
		Chart1.Series.Clear()
		Chart1.Titles.Clear()
		Chart1.Titles.Add("Wetterdaten für den Monat : " & formattedDate)
		Chart1.Series.Add(chartSeries)
		' Combobox zurücksetzen
		cmb_Wettertyp.SelectedIndex = 1
	End Sub

	Private Sub cmb_Wettertyp_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cmb_Wettertyp.SelectedIndexChanged
		LoadWeatherDataAndChart()
	End Sub

	Private Sub LoadWeatherDataAndChart()
		ReadDataFromXml()
		'Datum im Format "yyyy-MM" aus dem DTPicker auslesen
		Dim selectedDate As String = dtp_Show.Value.ToString("yyyy-MM")
		'Pfad zur ausgewählten XML-Datei erstellen
		Dim xmlFilePath As String = Path.Combine(NewDatafolder, selectedDate & ".xml")
		'Prüfen, ob die ausgewählte XML-Datei existiert
		If File.Exists(xmlFilePath) Then
			Dim weatherDataList As List(Of WeatherDataDiag) = ReadWeatherDataFromXml(xmlFilePath)
			' Chart-Serie erstellen, wenn ein Datentyp ausgewählt wurde
			If cmb_Wettertyp.SelectedItem IsNot Nothing Then
				' Chart-Serie erstellen
				Dim chartSeries As New Series(cmb_Wettertyp.SelectedItem.ToString())
				Dim chartSeries1 As Series = Nothing ' Neue Serie erstellen
				Select Case cmb_Wettertyp.SelectedItem.ToString()
					Case "Regen"
						lblEinheiten.Text = "Regen: Niederschläge in mm"
						Chart1.ChartAreas(0).AxisX.Minimum = Double.NaN
						Chart1.ChartAreas(0).AxisX.Maximum = Double.NaN
						Chart1.ChartAreas(0).AxisY.Minimum = Double.NaN
						Chart1.ChartAreas(0).AxisY.Maximum = Double.NaN
						chartSeries.Color = Color.Blue
						chartSeries.ChartType = SeriesChartType.Column
						For Each weatherData As WeatherDataDiag In weatherDataList
							chartSeries.Points.AddXY(weatherData.Datum, weatherData.RegenTag)
						Next
					Case "SchneeTag"
						lblEinheiten.Text = "Schnee: Niederschläge in cm"
						Chart1.ChartAreas(0).AxisX.Minimum = Double.NaN
						Chart1.ChartAreas(0).AxisX.Maximum = Double.NaN
						Chart1.ChartAreas(0).AxisY.Minimum = Double.NaN
						Chart1.ChartAreas(0).AxisY.Maximum = Double.NaN
						chartSeries.Color = Color.DarkSlateBlue
						chartSeries.ChartType = SeriesChartType.Column
						chartSeries1 = New Series("SchneeNacht") ' Neue Serie initialisieren
						chartSeries1.Color = Color.Magenta
						chartSeries1.ChartType = SeriesChartType.Column
						For Each weatherData As WeatherDataDiag In weatherDataList
							chartSeries.Points.AddXY(weatherData.Datum, weatherData.SchneeTag)
							chartSeries1.Points.AddXY(weatherData.Datum, weatherData.SchneeNacht)
						Next
					Case "TempTag"
						lblEinheiten.Text = "Temperaturen in °Celsius"
						Chart1.ChartAreas(0).AxisX.Minimum = Double.NaN
						Chart1.ChartAreas(0).AxisX.Maximum = Double.NaN
						Chart1.ChartAreas(0).AxisY.Minimum = Double.NaN
						Chart1.ChartAreas(0).AxisY.Maximum = Double.NaN
						chartSeries.Color = Color.Maroon
						chartSeries.ChartType = SeriesChartType.Line
						chartSeries1 = New Series("TempNacht") ' Neue Serie initialisieren
						chartSeries1.Color = Color.DarkGreen
						chartSeries1.ChartType = SeriesChartType.Line
						For Each weatherData As WeatherDataDiag In weatherDataList
							chartSeries.Points.AddXY(weatherData.Datum, weatherData.TempTag)
							chartSeries1.Points.AddXY(weatherData.Datum, weatherData.TempNacht)
						Next
					Case "WindTag"
						lblEinheiten.Text = "Windstärke nach Beaufort-Skala"
						Chart1.ChartAreas(0).AxisX.Minimum = Double.NaN
						Chart1.ChartAreas(0).AxisX.Maximum = Double.NaN
						Chart1.ChartAreas(0).AxisY.Minimum = Double.NaN
						Chart1.ChartAreas(0).AxisY.Maximum = Double.NaN
						chartSeries.Color = Color.MediumVioletRed
						chartSeries.ChartType = SeriesChartType.Line
						chartSeries1 = New Series("WindNacht") ' Neue Serie initialisieren
						chartSeries1.Color = Color.DarkSlateGray
						chartSeries1.ChartType = SeriesChartType.Line
						For Each weatherData As WeatherDataDiag In weatherDataList
							chartSeries.Points.AddXY(weatherData.Datum, weatherData.WindTag)
							chartSeries1.Points.AddXY(weatherData.Datum, weatherData.WindNacht)
						Next
					Case "LuftfmaxTag"
						lblEinheiten.Text = "Luftfeuchtigkeit in %"
						Chart1.ChartAreas(0).AxisX.Minimum = Double.NaN
						Chart1.ChartAreas(0).AxisX.Maximum = Double.NaN
						Chart1.ChartAreas(0).AxisY.Minimum = Double.NaN
						Chart1.ChartAreas(0).AxisY.Maximum = Double.NaN
						chartSeries.Color = Color.DeepPink
						chartSeries.ChartType = SeriesChartType.Line
						chartSeries1 = New Series("LuftfminTag") ' Neue Serie initialisieren
						chartSeries1.Color = Color.LimeGreen
						chartSeries1.ChartType = SeriesChartType.Line
						For Each weatherData As WeatherDataDiag In weatherDataList
							chartSeries.Points.AddXY(weatherData.Datum, weatherData.LuftfminTag)
							chartSeries1.Points.AddXY(weatherData.Datum, weatherData.LuftfmaxTag)
						Next
					Case "LuftfmaxNacht"
						lblEinheiten.Text = "Luftfeuchtigkeit in %"
						Chart1.ChartAreas(0).AxisX.Minimum = Double.NaN
						Chart1.ChartAreas(0).AxisX.Maximum = Double.NaN
						Chart1.ChartAreas(0).AxisY.Minimum = Double.NaN
						Chart1.ChartAreas(0).AxisY.Maximum = Double.NaN
						chartSeries.Color = Color.DarkGreen
						chartSeries.ChartType = SeriesChartType.Line
						chartSeries1 = New Series("LuftfminNacht") ' Neue Serie initialisieren
						chartSeries1.Color = Color.OrangeRed
						chartSeries1.ChartType = SeriesChartType.Line
						For Each weatherData As WeatherDataDiag In weatherDataList
							chartSeries.Points.AddXY(weatherData.Datum, weatherData.LuftfminNacht)
							chartSeries1.Points.AddXY(weatherData.Datum, weatherData.LuftfmaxNacht)
						Next
				End Select
				' Chart mit den Serien füllen
				Dim selectedDatein As Date = dtp_Show.Value
				Dim formattedDate As String = selectedDatein.ToString("MMMM-yyyy")
				Chart1.Series.Clear()
				Chart1.Titles.Clear()
				Chart1.ChartAreas.Clear() ' Alle ChartAreas löschen
				Chart1.Titles.Add("Wetterdaten für den Monat : " & formattedDate)
				' Neue ChartArea initialisieren
				Dim chartArea1 As New ChartArea()
				chartArea1.AxisX.IntervalType = DateTimeIntervalType.Days ' Achsenintervalltyp auf Tage setzen
				chartArea1.AxisX.Interval = 1 ' Achsenintervall auf 1 Tag setzen
				chartArea1.AxisX.LabelStyle.Format = "dd.MM" ' Datumsformat für die X-Achse setzen
				'  chartArea1.AxisY.Title = cmb_Wettertyp.SelectedItem.ToString()
				Chart1.ChartAreas.Add(chartArea1) ' ChartArea hinzufügen
				' Chart mit den Serien füllen
				Chart1.Series.Add(chartSeries)
				If chartSeries1 IsNot Nothing Then ' Sicherstellen, dass die neue Serie initialisiert wurde
					Chart1.Series.Add(chartSeries1)
				End If

			End If
		Else
			MessageBox.Show("Die ausgewählte Datei wurde nicht gefunden.", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error)
		End If
	End Sub
#End Region


#Region "Chart Anzeige wechseln"
	Private Sub btnCharstyle_Click(sender As Object, e As EventArgs) Handles btnCharstyle.Click
		For Each series As Series In Chart1.Series
			If series.ChartType = SeriesChartType.Column Then
				series.ChartType = SeriesChartType.Line
			Else
				series.ChartType = SeriesChartType.Column
			End If
		Next
	End Sub
#End Region


#Region "Form_Load"
	Private Sub frmDiagramm_Load(sender As Object, e As EventArgs) Handles Me.Load
		dtp_Show.CustomFormat = "MM/yyyy"
		dtp_Show.Format = DateTimePickerFormat.Custom
		ReadDataFromXml() : Comboboxintialisieren()
		'Datum im Format "yyyy-MM" aus dem DTPicker auslesen
		Dim selectedDate As String = dtp_Show.Value.ToString("yyyy-MM")
		'Pfad zur ausgewählten XML-Datei erstellen
		Dim xmlFilePath As String = Path.Combine(NewDatafolder, selectedDate & ".xml")
		'Prüfen, ob die ausgewählte XML-Datei existiert
		If File.Exists(xmlFilePath) Then
			lblEinheiten.Text = "Temperaturen in °Celsius"
			Dim weatherDataList As List(Of WeatherDataDiag) = ReadWeatherDataFromXml(xmlFilePath)
			' -Serie erstellen
			Dim TempTSerie As New Series("Temperatur Tag")
			TempTSerie.ChartType = SeriesChartType.Line
			TempTSerie.Color = Color.Maroon
			' -Serie erstellen
			Dim TempNSerie As New Series("Temperatur Nacht")
			TempNSerie.ChartType = SeriesChartType.Line
			TempNSerie.Color = Color.ForestGreen
			For Each weatherData As WeatherDataDiag In weatherDataList
				TempTSerie.Points.AddXY(weatherData.Datum, weatherData.TempTag)
				TempNSerie.Points.AddXY(weatherData.Datum, weatherData.TempNacht)
			Next
			' Chart mit den Serien füllen
			Dim selectedDatein As Date = dtp_Show.Value
			Dim formattedDate As String = selectedDatein.ToString("MMMM-yyyy")
			Chart1.Series.Clear()
			Chart1.Titles.Clear()
			Chart1.Titles.Add("Wetterdaten für den Monat : " & formattedDate)
			Chart1.Series.Add(TempTSerie)
			Chart1.Series.Add(TempNSerie)
		Else
			MessageBox.Show("Die ausgewählte Datei wurde nicht gefunden.", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error)
		End If
	End Sub

	Private Sub Comboboxintialisieren()
		cmb_Wettertyp.Items.Clear()
		'ComboBox mit Standardwert ("Bitte Auswählen") initialisieren
		cmb_Wettertyp.Items.Add("Bitte Auswählen")
		cmb_Wettertyp.SelectedIndex = 0 'Standardwert auswählen
		'weitere ComboBox-Elemente hinzufügen
		cmb_Wettertyp.Items.Add("Regen")
		cmb_Wettertyp.Items.Add("SchneeTag")
		cmb_Wettertyp.Items.Add("TempTag")
		cmb_Wettertyp.Items.Add("WindTag")
		cmb_Wettertyp.Items.Add("LuftfmaxTag")
		cmb_Wettertyp.Items.Add("LuftfmaxNacht")
		'DropDownStyle auf DropDownList setzen
		cmb_Wettertyp.DropDownStyle = ComboBoxStyle.DropDownList
	End Sub
#End Region


#Region "Drucken"
	'----------------------
	Private Sub chkPrint_CheckedChanged(sender As Object, e As EventArgs) Handles chkPrint.CheckedChanged
		AktualisierePaper()
	End Sub

	Private Sub AktualisierePaper()
		If chkPrint.Checked Then
			Paperformat = True
		Else
			Paperformat = False
		End If
	End Sub

	Private Sub PrintChartButton_Click(sender As Object, e As EventArgs) Handles btnPrint.Click
		' Create a PrintDocument object
		Dim pd As New Printing.PrintDocument()
		' Set the printer settings
		pd.DefaultPageSettings.Landscape = Paperformat ' Set the page orientation to landscape
		pd.DefaultPageSettings.PaperSize = New PaperSize("A4", 827, 1169) ' Set the paper size to A4
		' Add an event handler to print the chart
		AddHandler pd.PrintPage, AddressOf PrintChart
		' Print the chart
		pd.Print()
	End Sub

	Private Sub PrintChart(sender As Object, e As Printing.PrintPageEventArgs)
		' Draw the chart on a bitmap image
		' Calculate the dimensions of the chart on the printed page
		Dim RowIndex = 0
		For i = 0 To Images.Count - 1
			Dim chartWidth As Integer = CInt(225 * 1.5)
			Dim chartHeight As Integer = CInt(150 * 1.5)
			' Draw the chart on the printed page
			e.Graphics.DrawImage(Images(i), CInt(e.MarginBounds.Left + 250 * 1.5 * (i Mod 2)), CInt(e.MarginBounds.Top + (175 * 1.5 * RowIndex)), chartWidth, chartHeight)
			If i Mod 2 = 1 Then RowIndex += 1
		Next
	End Sub
#End Region


#Region "Funktionen"
	Private Function ReadWeatherDataFromXml(xmlFileName As String) As List(Of WeatherDataDiag)
		Dim weatherDataList As New List(Of WeatherDataDiag)
		Dim doc As New XmlDocument()
		doc.Load(xmlFileName)
		For Each node As XmlNode In doc.DocumentElement.ChildNodes
			Dim WeatherDataDiag As New WeatherDataDiag()
			WeatherDataDiag.Datum = DateTime.Parse(node.SelectSingleNode("Datum").InnerText)
			WeatherDataDiag.WetterTag = node.SelectSingleNode("WetterTag").InnerText
			WeatherDataDiag.WetterNacht = node.SelectSingleNode("WetterNacht").InnerText
			WeatherDataDiag.RegenTag = Double.Parse(node.SelectSingleNode("RegenTag").InnerText)
			'WeatherDataDiag.RegenNacht = Double.Parse(node.SelectSingleNode("RegenNacht").InnerText)
			WeatherDataDiag.SchneeTag = Double.Parse(node.SelectSingleNode("SchneeTag").InnerText)
			WeatherDataDiag.SchneeNacht = Double.Parse(node.SelectSingleNode("SchneeNacht").InnerText)
			WeatherDataDiag.LuftfminTag = Double.Parse(node.SelectSingleNode("LuftfminTag").InnerText)
			WeatherDataDiag.LuftfmaxTag = Double.Parse(node.SelectSingleNode("LuftfmaxTag").InnerText)
			WeatherDataDiag.LuftfminNacht = Double.Parse(node.SelectSingleNode("LuftfminNacht").InnerText)
			WeatherDataDiag.LuftfmaxNacht = Double.Parse(node.SelectSingleNode("LuftfmaxNacht").InnerText)
			WeatherDataDiag.TempTag = Double.Parse(node.SelectSingleNode("TempTag").InnerText)
			WeatherDataDiag.TempNacht = Double.Parse(node.SelectSingleNode("TempNacht").InnerText)
			WeatherDataDiag.WindTag = Double.Parse(node.SelectSingleNode("WindTag").InnerText)
			WeatherDataDiag.WindNacht = Double.Parse(node.SelectSingleNode("WindNacht").InnerText)
			weatherDataList.Add(WeatherDataDiag)
		Next
		Return weatherDataList
	End Function

	Private Function GetWeatherDataList(selectedDate As Date) As List(Of WeatherDataDiag)
		'Datum im Format "yyyy-MM" aus dem DTPicker auslesen
		Dim dateString As String = selectedDate.ToString("yyyy-MM")
		'Pfad zur ausgewählten XML-Datei erstellen
		Dim xmlFilePath As String = Path.Combine(NewDatafolder, dateString & ".xml")
		'Prüfen, ob die ausgewählte XML-Datei existiert
		If File.Exists(xmlFilePath) Then
			Return ReadWeatherDataFromXml(xmlFilePath)
		Else
			MessageBox.Show("Die ausgewählte Datei wurde nicht gefunden.", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error)
			Return Nothing
		End If
	End Function

	Private Async Sub Button1_Click(sender As Object, e As EventArgs) Handles Button1.Click
		For i = 1 To cmb_Wettertyp.Items.Count - 1
			cmb_Wettertyp.SelectedIndex = i
			Await Threading.Tasks.Task.Delay(150)
			Dim chartImage As New Bitmap(Chart1.Width, Chart1.Height)
			Chart1.DrawToBitmap(chartImage, New Rectangle(0, 0, Chart1.Width, Chart1.Height))
			Images.Add(chartImage)
		Next
		PrintChartButton_Click(Me, EventArgs.Empty)
	End Sub
#End Region

End Class

Public Class WeatherDataDiag
	Public Wettertyp As String
	Public Property Datum As DateTime
	Public Property WetterTag As String
	Public Property WetterNacht As String
	Public Property RegenTag As Double
	'Public Property RegenNacht As Double
	Public Property SchneeTag As Double
	Public Property SchneeNacht As Double
	Public Property LuftfminTag As Double
	Public Property LuftfmaxTag As Double
	Public Property LuftfminNacht As Double
	Public Property LuftfmaxNacht As Double
	Public Property TempTag As Double
	Public Property TempNacht As Double
	Public Property WindTag As Double
	Public Property WindNacht As Double
End Class
