﻿Public Class frmSettings
    Private ReadOnly _appfolder As String = Path.Combine(Application.StartupPath, "ApSet.xml")
    Private ReadOnly _currentYear As Integer = DateTime.Now.Year

    Public Sub New()
        InitializeComponent()
        txt_Appfolder.Text = _appfolder
    End Sub

    Private Sub frmSettings_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        If Not File.Exists(_appfolder) Then
            MessageBox.Show("Die Konfigurationsdatei existiert nicht.", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error)
            Return
        End If

        Try
            Dim doc As New XmlDocument()
            doc.Load(_appfolder)

            Dim root As XmlElement = doc.DocumentElement
            If root.SelectSingleNode("username") IsNot Nothing AndAlso Not String.IsNullOrEmpty(root.SelectSingleNode("username").InnerText) Then
                txt_Username.Text = root.SelectSingleNode("username").InnerText
            Else
                MessageBox.Show("Der Benutzername ist ungültig.", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error)
            End If

            If root.SelectSingleNode("location") IsNot Nothing AndAlso Not String.IsNullOrEmpty(root.SelectSingleNode("location").InnerText) Then
                txt_Location.Text = root.SelectSingleNode("location").InnerText
            Else
                MessageBox.Show("Der Standort ist ungültig.", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error)
            End If

            If root.SelectSingleNode("appFolder") IsNot Nothing AndAlso Not String.IsNullOrEmpty(root.SelectSingleNode("appFolder").InnerText) Then
                txt_Appfolder.Text = root.SelectSingleNode("appFolder").InnerText
            Else
                MessageBox.Show("Der Anwendungsordner ist ungültig.", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error)
            End If

            If root.SelectSingleNode("dataFolder") IsNot Nothing AndAlso Not String.IsNullOrEmpty(root.SelectSingleNode("dataFolder").InnerText) Then
                txt_Datafolder.Text = root.SelectSingleNode("dataFolder").InnerText
                lbl_Datafolder.Text = txt_Datafolder.Text
            Else
                MessageBox.Show("Der Datenordner ist ungültig.", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error)
            End If
        Catch ex As Exception
            MessageBox.Show("Beim Laden der Konfigurationsdatei ist ein Fehler aufgetreten: " & ex.Message, "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub


    Private Sub BtnFolder_Click(sender As Object, e As EventArgs) Handles btnFolder.Click
        Try
            Dim folderDialog As New FolderBrowserDialog()
            ' Zeige den Dialog an und warte auf das Ergebnis
            Dim result As DialogResult = folderDialog.ShowDialog()
            If result <> DialogResult.OK Then
                Return
            End If
            Dim selectedFolder = folderDialog.SelectedPath
            Dim selectedFolderInfo As New DirectoryInfo(selectedFolder)
            ' Prüfe, ob die Checkbox angeklickt wurde
            If chk_MakeFolder.Checked Then
                ' Erstelle ein neues Verzeichnis mit dem Namen "WetterApp", falls es noch nicht vorhanden ist
                Dim weatherAppFolder As New DirectoryInfo(Path.Combine(selectedFolder, "WetterApp"))
                If Not weatherAppFolder.Exists Then
                    weatherAppFolder.Create()
                End If
                ' Erstelle ein neues Verzeichnis mit dem Namen "WetterApp\aktl Jahr", falls es noch nicht vorhanden ist
                Dim currentYearFolder As New DirectoryInfo(Path.Combine(weatherAppFolder.FullName, _currentYear.ToString()))
                If Not currentYearFolder.Exists Then
                    currentYearFolder.Create()
                End If
                selectedFolder = currentYearFolder.FullName
            End If
            ' Ordnerpfad in Textbox und Label eintragen
            txt_Datafolder.Text = selectedFolder
            lbl_Datafolder.Text = selectedFolder
        Catch ex As Exception
            ' Zeige eine Fehlermeldung an, wenn beim Aufrufen des Ordnerauswahldialogs ein Fehler auftritt
            MessageBox.Show("Beim Öffnen des Ordnerauswahldialogs ist ein Fehler aufgetreten: " & ex.Message, "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Private Sub btnSave_Click(sender As Object, e As EventArgs) Handles btn_Save.Click
        Dim settings As New AppSettings(txt_Username.Text, txt_Location.Text, txt_Appfolder.Text, txt_Datafolder.Text)
        Try
            ' Überprüfen, ob die Textfelder ausgefüllt sind
            If Not settings.IsValid() Then
                MessageBox.Show("Bitte füllen Sie die Pflichtfelder aus " & Environment.NewLine & "und wählen Sie ein Verzeichnis " & Environment.NewLine & "indem die Daten gespeichert werden sollen.", "Fehlende Daten", MessageBoxButtons.OK, MessageBoxIcon.Error)
                Return
            End If

            ' XML-Datei schreiben
            Dim fileName As String = Path.Combine(Application.StartupPath, "ApSet.xml")
            Dim xmlWriterSettings As New XmlWriterSettings()
            xmlWriterSettings.Indent = True
            Using writer As XmlWriter = XmlWriter.Create(fileName, xmlWriterSettings)
                writer.WriteStartDocument()
                writer.WriteStartElement("appSettings")
                writer.WriteElementString("username", settings.Username)
                writer.WriteElementString("location", settings.Location)
                writer.WriteElementString("appFolder", settings.AppFolder)
                writer.WriteElementString("dataFolder", settings.DataFolder)
                writer.WriteEndElement()
                writer.WriteEndDocument()
            End Using
            MessageBox.Show("Die Einstellungen wurden erfolgreich gespeichert.", "Erfolgreich gespeichert", MessageBoxButtons.OK, MessageBoxIcon.Information)
        Catch ex As Exception
            MessageBox.Show("Fehler beim Speichern der Einstellungen. " & ex.Message, "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Private Sub btn_Close_Click(sender As Object, e As EventArgs) Handles btn_Close.Click
        Me.Close()
    End Sub

    Public Class AppSettings
        Public Property Username As String
        Public Property Location As String
        Public Property Appfolder As String
        Public Property Datafolder As String

        Public Sub New(username As String, location As String, appfolder As String, datafolder As String)
            Me.Username = username
            Me.Location = location
            Me.Appfolder = appfolder
            Me.Datafolder = datafolder
        End Sub

        Public Function IsValid() As Boolean
            Return Not String.IsNullOrEmpty(Username) AndAlso Not String.IsNullOrEmpty(Location) AndAlso Not String.IsNullOrEmpty(Datafolder)
        End Function
    End Class

End Class